@@ -59,6 +59,7 @@ __all__ = [
     "get_traceparent",
     "is_initialized",
     "isolation_scope",
+    "last_event_id",
     "new_scope",
     "push_scope",
     "set_context",
@@ -332,6 +333,16 @@ def start_transaction(
     )
 
 
+@scopemethod
+def last_event_id():
+    # type: () -> Optional[str]
+    """
+    See :py:meth:`sentry_sdk.Scope.last_event_id` documentation regarding
+    this method's limitations.
+    """
+    return Scope.last_event_id()
+
+
 def set_measurement(name, value, unit=""):
     # type: (str, float, MeasurementUnit) -> None
     transaction = Scope.get_current_scope().transaction
