@@ -1,10 +1,8 @@
-import copy
-import sys
 from contextlib import contextmanager
 
 from sentry_sdk._compat import with_metaclass
 from sentry_sdk.consts import INSTRUMENTER
-from sentry_sdk.scope import Scope
+from sentry_sdk.scope import Scope, _ScopeManager
 from sentry_sdk.client import Client
 from sentry_sdk.tracing import (
     NoOpSpan,
@@ -33,6 +31,9 @@ if TYPE_CHECKING:
     from typing import TypeVar
     from typing import Union
 
+    from typing_extensions import Unpack
+
+    from sentry_sdk.client import BaseClient
     from sentry_sdk.integrations import Integration
     from sentry_sdk._types import (
         Event,
@@ -41,8 +42,10 @@ if TYPE_CHECKING:
         BreadcrumbHint,
         ExcInfo,
         LogLevelStr,
+        SamplingContext,
     )
     from sentry_sdk.consts import ClientConstructor
+    from sentry_sdk.tracing import TransactionKwargs
 
     T = TypeVar("T")
 
@@ -58,13 +61,15 @@ _local = ContextVar("sentry_current_hub")
 
 def _should_send_default_pii():
     # type: () -> bool
+    # TODO: Migrate existing code to `scope.should_send_default_pii()` and remove this function.
+    # New code should not use this function!
     client = Hub.current.client
     if not client:
         return False
-    return client.options["send_default_pii"]
+    return client.should_send_default_pii()
 
 
-class _InitGuard(object):
+class _InitGuard:
     def __init__(self, client):
         # type: (Client) -> None
         self._client = client
@@ -82,16 +87,10 @@ class _InitGuard(object):
 
 def _check_python_deprecations():
     # type: () -> None
-    version = sys.version_info[:2]
-
-    if version == (3, 4) or version == (3, 5):
-        logger.warning(
-            "sentry-sdk 2.0.0 will drop support for Python %s.",
-            "{}.{}".format(*version),
-        )
-        logger.warning(
-            "Please upgrade to the latest version to continue receiving upgrades and bugfixes."
-        )
+    # Since we're likely to deprecate Python versions in the future, I'm keeping
+    # this handy function around. Use this to detect the Python version used and
+    # to output logger.warning()s if it's deprecated.
+    pass
 
 
 def _init(*args, **kwargs):
@@ -145,56 +144,12 @@ class HubMeta(type):
         return GLOBAL_HUB
 
 
-class _ScopeManager(object):
-    def __init__(self, hub):
-        # type: (Hub) -> None
-        self._hub = hub
-        self._original_len = len(hub._stack)
-        self._layer = hub._stack[-1]
-
-    def __enter__(self):
-        # type: () -> Scope
-        scope = self._layer[1]
-        assert scope is not None
-        return scope
-
-    def __exit__(self, exc_type, exc_value, tb):
-        # type: (Any, Any, Any) -> None
-        current_len = len(self._hub._stack)
-        if current_len < self._original_len:
-            logger.error(
-                "Scope popped too soon. Popped %s scopes too many.",
-                self._original_len - current_len,
-            )
-            return
-        elif current_len > self._original_len:
-            logger.warning(
-                "Leaked %s scopes: %s",
-                current_len - self._original_len,
-                self._hub._stack[self._original_len :],
-            )
-
-        layer = self._hub._stack[self._original_len - 1]
-        del self._hub._stack[self._original_len - 1 :]
-
-        if layer[1] != self._layer[1]:
-            logger.error(
-                "Wrong scope found. Meant to pop %s, but popped %s.",
-                layer[1],
-                self._layer[1],
-            )
-        elif layer[0] != self._layer[0]:
-            warning = (
-                "init() called inside of pushed scope. This might be entirely "
-                "legitimate but usually occurs when initializing the SDK inside "
-                "a request handler or task/job function. Try to initialize the "
-                "SDK as early as possible instead."
-            )
-            logger.warning(warning)
-
-
 class Hub(with_metaclass(HubMeta)):  # type: ignore
-    """The hub wraps the concurrency management of the SDK.  Each thread has
+    """
+    .. deprecated:: 2.0.0
+        The Hub is deprecated. Its functionality will be merged into :py:class:`sentry_sdk.scope.Scope`.
+
+    The hub wraps the concurrency management of the SDK.  Each thread has
     its own hub but the hub might transfer with the flow of execution if
     context vars are available.
 
@@ -202,6 +157,7 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
     """
 
     _stack = None  # type: List[Tuple[Optional[Client], Scope]]
+    _scope = None  # type: Optional[Scope]
 
     # Mypy doesn't pick up on the metaclass.
 
@@ -215,24 +171,50 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
         scope=None,  # type: Optional[Any]
     ):
         # type: (...) -> None
-        if isinstance(client_or_hub, Hub):
-            hub = client_or_hub
-            client, other_scope = hub._stack[-1]
-            if scope is None:
-                scope = copy.copy(other_scope)
-        else:
-            client = client_or_hub
-        if scope is None:
-            scope = Scope()
 
-        self._stack = [(client, scope)]
+        current_scope = None
+
+        if isinstance(client_or_hub, Hub):
+            client = Scope.get_client()
+            if scope is None:
+                # hub cloning is going on, we use a fork of the current/isolation scope for context manager
+                scope = Scope.get_isolation_scope().fork()
+                current_scope = Scope.get_current_scope().fork()
+        else:
+            client = client_or_hub  # type: ignore
+            Scope.get_global_scope().set_client(client)
+
+        if scope is None:  # so there is no Hub cloning going on
+            # just the current isolation scope is used for context manager
+            scope = Scope.get_isolation_scope()
+            current_scope = Scope.get_current_scope()
+
+        if current_scope is None:
+            # just the current current scope is used for context manager
+            current_scope = Scope.get_current_scope()
+
+        self._stack = [(client, scope)]  # type: ignore
         self._last_event_id = None  # type: Optional[str]
         self._old_hubs = []  # type: List[Hub]
 
+        self._old_current_scopes = []  # type: List[Scope]
+        self._old_isolation_scopes = []  # type: List[Scope]
+        self._current_scope = current_scope  # type: Scope
+        self._scope = scope  # type: Scope
+
     def __enter__(self):
         # type: () -> Hub
         self._old_hubs.append(Hub.current)
         _local.set(self)
+
+        current_scope = Scope.get_current_scope()
+        self._old_current_scopes.append(current_scope)
+        scope._current_scope.set(self._current_scope)
+
+        isolation_scope = Scope.get_isolation_scope()
+        self._old_isolation_scopes.append(isolation_scope)
+        scope._isolation_scope.set(self._scope)
+
         return self
 
     def __exit__(
@@ -245,11 +227,21 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
         old = self._old_hubs.pop()
         _local.set(old)
 
+        old_current_scope = self._old_current_scopes.pop()
+        scope._current_scope.set(old_current_scope)
+
+        old_isolation_scope = self._old_isolation_scopes.pop()
+        scope._isolation_scope.set(old_isolation_scope)
+
     def run(
         self, callback  # type: Callable[[], T]
     ):
         # type: (...) -> T
-        """Runs a callback in the context of the hub.  Alternatively the
+        """
+        .. deprecated:: 2.0.0
+            This function is deprecated and will be removed in a future release.
+
+        Runs a callback in the context of the hub.  Alternatively the
         with statement can be used on the hub directly.
         """
         with self:
@@ -259,28 +251,46 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
         self, name_or_class  # type: Union[str, Type[Integration]]
     ):
         # type: (...) -> Any
-        """Returns the integration for this hub by name or class.  If there
+        """
+        .. deprecated:: 2.0.0
+            This function is deprecated and will be removed in a future release.
+            Please use :py:meth:`sentry_sdk.client._Client.get_integration` instead.
+
+        Returns the integration for this hub by name or class.  If there
         is no client bound or the client does not have that integration
         then `None` is returned.
 
         If the return value is not `None` the hub is guaranteed to have a
         client attached.
         """
-        client = self.client
-        if client is not None:
-            return client.get_integration(name_or_class)
+        return Scope.get_client().get_integration(name_or_class)
 
     @property
     def client(self):
-        # type: () -> Optional[Client]
-        """Returns the current client on the hub."""
-        return self._stack[-1][0]
+        # type: () -> Optional[BaseClient]
+        """
+        .. deprecated:: 2.0.0
+            This property is deprecated and will be removed in a future release.
+            Please use :py:func:`sentry_sdk.api.get_client` instead.
+
+        Returns the current client on the hub.
+        """
+        client = Scope.get_client()
+
+        if not client.is_active():
+            return None
+
+        return client
 
     @property
     def scope(self):
         # type: () -> Scope
-        """Returns the current scope on the hub."""
-        return self._stack[-1][1]
+        """
+        .. deprecated:: 2.0.0
+            This property is deprecated and will be removed in a future release.
+            Returns the current scope on the hub.
+        """
+        return Scope.get_isolation_scope()
 
     def last_event_id(self):
         # type: () -> Optional[str]
@@ -296,16 +306,25 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
         return self._last_event_id
 
     def bind_client(
-        self, new  # type: Optional[Client]
+        self, new  # type: Optional[BaseClient]
     ):
         # type: (...) -> None
-        """Binds a new client to the hub."""
-        top = self._stack[-1]
-        self._stack[-1] = (new, top[1])
+        """
+        .. deprecated:: 2.0.0
+            This function is deprecated and will be removed in a future release.
+            Please use :py:meth:`sentry_sdk.Scope.set_client` instead.
+
+        Binds a new client to the hub.
+        """
+        Scope.get_global_scope().set_client(new)
 
     def capture_event(self, event, hint=None, scope=None, **scope_kwargs):
         # type: (Event, Optional[Hint], Optional[Scope], Any) -> Optional[str]
         """
+        .. deprecated:: 2.0.0
+            This function is deprecated and will be removed in a future release.
+            Please use :py:meth:`sentry_sdk.Scope.capture_event` instead.
+
         Captures an event.
 
         Alias of :py:meth:`sentry_sdk.Scope.capture_event`.
@@ -321,12 +340,8 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
             For supported `**scope_kwargs` see :py:meth:`sentry_sdk.Scope.update_from_kwargs`.
             The `scope` and `scope_kwargs` parameters are mutually exclusive.
         """
-        client, top_scope = self._stack[-1]
-        if client is None:
-            return None
-
-        last_event_id = top_scope.capture_event(
-            event, hint, client=client, scope=scope, **scope_kwargs
+        last_event_id = Scope.get_current_scope().capture_event(
+            event, hint, scope=scope, **scope_kwargs
         )
 
         is_transaction = event.get("type") == "transaction"
@@ -338,6 +353,10 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
     def capture_message(self, message, level=None, scope=None, **scope_kwargs):
         # type: (str, Optional[LogLevelStr], Optional[Scope], Any) -> Optional[str]
         """
+        .. deprecated:: 2.0.0
+            This function is deprecated and will be removed in a future release.
+            Please use :py:meth:`sentry_sdk.Scope.capture_message` instead.
+
         Captures a message.
 
         Alias of :py:meth:`sentry_sdk.Scope.capture_message`.
@@ -353,14 +372,10 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
             For supported `**scope_kwargs` see :py:meth:`sentry_sdk.Scope.update_from_kwargs`.
             The `scope` and `scope_kwargs` parameters are mutually exclusive.
 
-        :returns: An `event_id` if the SDK decided to send the event (see :py:meth:`sentry_sdk.Client.capture_event`).
+        :returns: An `event_id` if the SDK decided to send the event (see :py:meth:`sentry_sdk.client._Client.capture_event`).
         """
-        client, top_scope = self._stack[-1]
-        if client is None:
-            return None
-
-        last_event_id = top_scope.capture_message(
-            message, level=level, client=client, scope=scope, **scope_kwargs
+        last_event_id = Scope.get_current_scope().capture_message(
+            message, level=level, scope=scope, **scope_kwargs
         )
 
         if last_event_id is not None:
@@ -370,7 +385,12 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
 
     def capture_exception(self, error=None, scope=None, **scope_kwargs):
         # type: (Optional[Union[BaseException, ExcInfo]], Optional[Scope], Any) -> Optional[str]
-        """Captures an exception.
+        """
+        .. deprecated:: 2.0.0
+            This function is deprecated and will be removed in a future release.
+            Please use :py:meth:`sentry_sdk.Scope.capture_exception` instead.
+
+        Captures an exception.
 
         Alias of :py:meth:`sentry_sdk.Scope.capture_exception`.
 
@@ -383,14 +403,10 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
             For supported `**scope_kwargs` see :py:meth:`sentry_sdk.Scope.update_from_kwargs`.
             The `scope` and `scope_kwargs` parameters are mutually exclusive.
 
-        :returns: An `event_id` if the SDK decided to send the event (see :py:meth:`sentry_sdk.Client.capture_event`).
+        :returns: An `event_id` if the SDK decided to send the event (see :py:meth:`sentry_sdk.client._Client.capture_event`).
         """
-        client, top_scope = self._stack[-1]
-        if client is None:
-            return None
-
-        last_event_id = top_scope.capture_exception(
-            error, client=client, scope=scope, **scope_kwargs
+        last_event_id = Scope.get_current_scope().capture_exception(
+            error, scope=scope, **scope_kwargs
         )
 
         if last_event_id is not None:
@@ -403,10 +419,14 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
     ):
         # type: (...) -> Any
         """
+        .. deprecated:: 2.0.0
+            This function is deprecated and will be removed in a future release.
+            Please use :py:meth:`sentry_sdk.client._Client._capture_internal_exception` instead.
+
         Capture an exception that is likely caused by a bug in the SDK
         itself.
 
-        Duplicated in :py:meth:`sentry_sdk.Client._capture_internal_exception`.
+        Duplicated in :py:meth:`sentry_sdk.client._Client._capture_internal_exception`.
 
         These exceptions do not end up in Sentry and are just logged instead.
         """
@@ -415,6 +435,10 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
     def add_breadcrumb(self, crumb=None, hint=None, **kwargs):
         # type: (Optional[Breadcrumb], Optional[BreadcrumbHint], Any) -> None
         """
+        .. deprecated:: 2.0.0
+            This function is deprecated and will be removed in a future release.
+            Please use :py:meth:`sentry_sdk.Scope.add_breadcrumb` instead.
+
         Adds a breadcrumb.
 
         :param crumb: Dictionary with the data as the sentry v7/v8 protocol expects.
@@ -422,18 +446,15 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
         :param hint: An optional value that can be used by `before_breadcrumb`
             to customize the breadcrumbs that are emitted.
         """
-        client, scope = self._stack[-1]
-        if client is None:
-            logger.info("Dropped breadcrumb because no client bound")
-            return
+        Scope.get_isolation_scope().add_breadcrumb(crumb, hint, **kwargs)
 
-        kwargs["client"] = client
-
-        scope.add_breadcrumb(crumb, hint, **kwargs)
-
-    def start_span(self, span=None, instrumenter=INSTRUMENTER.SENTRY, **kwargs):
-        # type: (Optional[Span], str, Any) -> Span
+    def start_span(self, instrumenter=INSTRUMENTER.SENTRY, **kwargs):
+        # type: (str, Any) -> Span
         """
+        .. deprecated:: 2.0.0
+            This function is deprecated and will be removed in a future release.
+            Please use :py:meth:`sentry_sdk.Scope.start_span` instead.
+
         Start a span whose parent is the currently active span or transaction, if any.
 
         The return value is a :py:class:`sentry_sdk.tracing.Span` instance,
@@ -448,18 +469,22 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
 
         For supported `**kwargs` see :py:class:`sentry_sdk.tracing.Span`.
         """
-        client, scope = self._stack[-1]
-
-        kwargs["hub"] = self
-        kwargs["client"] = client
-
-        return scope.start_span(span=span, instrumenter=instrumenter, **kwargs)
+        scope = Scope.get_current_scope()
+        return scope.start_span(instrumenter=instrumenter, **kwargs)
 
     def start_transaction(
-        self, transaction=None, instrumenter=INSTRUMENTER.SENTRY, **kwargs
+        self,
+        transaction=None,
+        instrumenter=INSTRUMENTER.SENTRY,
+        custom_sampling_context=None,
+        **kwargs
     ):
-        # type: (Optional[Transaction], str, Any) -> Union[Transaction, NoOpSpan]
+        # type: (Optional[Transaction], str, Optional[SamplingContext], Unpack[TransactionKwargs]) -> Union[Transaction, NoOpSpan]
         """
+        .. deprecated:: 2.0.0
+            This function is deprecated and will be removed in a future release.
+            Please use :py:meth:`sentry_sdk.Scope.start_transaction` instead.
+
         Start and return a transaction.
 
         Start an existing transaction if given, otherwise create and start a new
@@ -483,23 +508,27 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
 
         For supported `**kwargs` see :py:class:`sentry_sdk.tracing.Transaction`.
         """
-        client, scope = self._stack[-1]
+        scope = Scope.get_current_scope()
 
-        kwargs["hub"] = self
-        kwargs["client"] = client
+        # For backwards compatibility, we allow passing the scope as the hub.
+        # We need a major release to make this nice. (if someone searches the code: deprecated)
+        # Type checking disabled for this line because deprecated keys are not allowed in the type signature.
+        kwargs["hub"] = scope  # type: ignore
 
         return scope.start_transaction(
-            transaction=transaction, instrumenter=instrumenter, **kwargs
+            transaction, instrumenter, custom_sampling_context, **kwargs
         )
 
     def continue_trace(self, environ_or_headers, op=None, name=None, source=None):
         # type: (Dict[str, Any], Optional[str], Optional[str], Optional[str]) -> Transaction
         """
+        .. deprecated:: 2.0.0
+            This function is deprecated and will be removed in a future release.
+            Please use :py:meth:`sentry_sdk.Scope.continue_trace` instead.
+
         Sets the propagation context from environment or headers and returns a transaction.
         """
-        scope = self._stack[-1][1]
-
-        return scope.continue_trace(
+        return Scope.get_isolation_scope().continue_trace(
             environ_or_headers=environ_or_headers, op=op, name=name, source=source
         )
 
@@ -524,6 +553,9 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
     ):
         # type: (...) -> Optional[ContextManager[Scope]]
         """
+        .. deprecated:: 2.0.0
+            This function is deprecated and will be removed in a future release.
+
         Pushes a new layer on the scope stack.
 
         :param callback: If provided, this method pushes a scope, calls
@@ -537,21 +569,14 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
                 callback(scope)
             return None
 
-        client, scope = self._stack[-1]
-
-        new_scope = copy.copy(scope)
-
-        if continue_trace:
-            new_scope.generate_propagation_context()
-
-        new_layer = (client, new_scope)
-        self._stack.append(new_layer)
-
         return _ScopeManager(self)
 
     def pop_scope_unsafe(self):
         # type: () -> Tuple[Optional[Client], Scope]
         """
+        .. deprecated:: 2.0.0
+            This function is deprecated and will be removed in a future release.
+
         Pops a scope layer from the stack.
 
         Try to use the context manager :py:meth:`push_scope` instead.
@@ -580,33 +605,31 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
         continue_trace=True,  # type: bool
     ):
         # type: (...) -> Optional[ContextManager[Scope]]
-
         """
+        .. deprecated:: 2.0.0
+            This function is deprecated and will be removed in a future release.
+
         Reconfigures the scope.
 
         :param callback: If provided, call the callback with the current scope.
 
         :returns: If no callback is provided, returns a context manager that returns the scope.
         """
-
-        client, scope = self._stack[-1]
+        scope = Scope.get_isolation_scope()
 
         if continue_trace:
             scope.generate_propagation_context()
 
         if callback is not None:
-            if client is not None:
-                callback(scope)
+            # TODO: used to return None when client is None. Check if this changes behavior.
+            callback(scope)
 
             return None
 
         @contextmanager
         def inner():
             # type: () -> Generator[Scope, None, None]
-            if client is not None:
-                yield scope
-            else:
-                yield Scope()
+            yield scope
 
         return inner()
 
@@ -614,37 +637,54 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
         self, session_mode="application"  # type: str
     ):
         # type: (...) -> None
-        """Starts a new session."""
-        client, scope = self._stack[-1]
-        scope.start_session(
-            client=client,
+        """
+        .. deprecated:: 2.0.0
+            This function is deprecated and will be removed in a future release.
+            Please use :py:meth:`sentry_sdk.Scope.start_session` instead.
+
+        Starts a new session.
+        """
+        Scope.get_isolation_scope().start_session(
             session_mode=session_mode,
         )
 
     def end_session(self):
         # type: (...) -> None
-        """Ends the current session if there is one."""
-        client, scope = self._stack[-1]
-        scope.end_session(client=client)
+        """
+        .. deprecated:: 2.0.0
+            This function is deprecated and will be removed in a future release.
+            Please use :py:meth:`sentry_sdk.Scope.end_session` instead.
+
+        Ends the current session if there is one.
+        """
+        Scope.get_isolation_scope().end_session()
 
     def stop_auto_session_tracking(self):
         # type: (...) -> None
-        """Stops automatic session tracking.
+        """
+        .. deprecated:: 2.0.0
+            This function is deprecated and will be removed in a future release.
+            Please use :py:meth:`sentry_sdk.Scope.stop_auto_session_tracking` instead.
+
+        Stops automatic session tracking.
 
         This temporarily session tracking for the current scope when called.
         To resume session tracking call `resume_auto_session_tracking`.
         """
-        client, scope = self._stack[-1]
-        scope.stop_auto_session_tracking(client=client)
+        Scope.get_isolation_scope().stop_auto_session_tracking()
 
     def resume_auto_session_tracking(self):
         # type: (...) -> None
-        """Resumes automatic session tracking for the current scope if
+        """
+        .. deprecated:: 2.0.0
+            This function is deprecated and will be removed in a future release.
+            Please use :py:meth:`sentry_sdk.Scope.resume_auto_session_tracking` instead.
+
+        Resumes automatic session tracking for the current scope if
         disabled earlier.  This requires that generally automatic session
         tracking is enabled.
         """
-        scope = self._stack[-1][1]
-        scope.resume_auto_session_tracking()
+        Scope.get_isolation_scope().resume_auto_session_tracking()
 
     def flush(
         self,
@@ -653,27 +693,47 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
     ):
         # type: (...) -> None
         """
-        Alias for :py:meth:`sentry_sdk.Client.flush`
+        .. deprecated:: 2.0.0
+            This function is deprecated and will be removed in a future release.
+            Please use :py:meth:`sentry_sdk.client._Client.flush` instead.
+
+        Alias for :py:meth:`sentry_sdk.client._Client.flush`
         """
-        client, scope = self._stack[-1]
-        if client is not None:
-            return client.flush(timeout=timeout, callback=callback)
+        return Scope.get_client().flush(timeout=timeout, callback=callback)
 
     def get_traceparent(self):
         # type: () -> Optional[str]
         """
+        .. deprecated:: 2.0.0
+            This function is deprecated and will be removed in a future release.
+            Please use :py:meth:`sentry_sdk.Scope.get_traceparent` instead.
+
         Returns the traceparent either from the active span or from the scope.
         """
-        client, scope = self._stack[-1]
-        return scope.get_traceparent(client=client)
+        current_scope = Scope.get_current_scope()
+        traceparent = current_scope.get_traceparent()
+
+        if traceparent is None:
+            isolation_scope = Scope.get_isolation_scope()
+            traceparent = isolation_scope.get_traceparent()
+
+        return traceparent
 
     def get_baggage(self):
         # type: () -> Optional[str]
         """
+        .. deprecated:: 2.0.0
+            This function is deprecated and will be removed in a future release.
+            Please use :py:meth:`sentry_sdk.Scope.get_baggage` instead.
+
         Returns Baggage either from the active span or from the scope.
         """
-        client, scope = self._stack[-1]
-        baggage = scope.get_baggage(client=client)
+        current_scope = Scope.get_current_scope()
+        baggage = current_scope.get_baggage()
+
+        if baggage is None:
+            isolation_scope = Scope.get_isolation_scope()
+            baggage = isolation_scope.get_baggage()
 
         if baggage is not None:
             return baggage.serialize()
@@ -683,17 +743,25 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
     def iter_trace_propagation_headers(self, span=None):
         # type: (Optional[Span]) -> Generator[Tuple[str, str], None, None]
         """
+        .. deprecated:: 2.0.0
+            This function is deprecated and will be removed in a future release.
+            Please use :py:meth:`sentry_sdk.Scope.iter_trace_propagation_headers` instead.
+
         Return HTTP headers which allow propagation of trace data. Data taken
         from the span representing the request, if available, or the current
         span on the scope if not.
         """
-        client, scope = self._stack[-1]
-
-        return scope.iter_trace_propagation_headers(span=span, client=client)
+        return Scope.get_current_scope().iter_trace_propagation_headers(
+            span=span,
+        )
 
     def trace_propagation_meta(self, span=None):
         # type: (Optional[Span]) -> str
         """
+        .. deprecated:: 2.0.0
+            This function is deprecated and will be removed in a future release.
+            Please use :py:meth:`sentry_sdk.Scope.trace_propagation_meta` instead.
+
         Return meta tags which should be injected into HTML templates
         to allow propagation of trace information.
         """
@@ -702,9 +770,14 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
                 "The parameter `span` in trace_propagation_meta() is deprecated and will be removed in the future."
             )
 
-        client, scope = self._stack[-1]
-        return scope.trace_propagation_meta(span=span, client=client)
+        return Scope.get_current_scope().trace_propagation_meta(
+            span=span,
+        )
 
 
 GLOBAL_HUB = Hub()
 _local.set(GLOBAL_HUB)
+
+
+# Circular imports
+from sentry_sdk import scope
