@@ -1,22 +1,24 @@
 import sys
 from copy import deepcopy
-from datetime import timedelta
+from datetime import datetime, timedelta, timezone
 from os import environ
 
+import sentry_sdk
 from sentry_sdk.api import continue_trace
 from sentry_sdk.consts import OP
-from sentry_sdk.hub import Hub, _should_send_default_pii
+from sentry_sdk.integrations import Integration
+from sentry_sdk.integrations._wsgi_common import _filter_headers
+from sentry_sdk.scope import should_send_default_pii
 from sentry_sdk.tracing import TRANSACTION_SOURCE_COMPONENT
-from sentry_sdk._compat import datetime_utcnow, duration_in_milliseconds, reraise
 from sentry_sdk.utils import (
     AnnotatedValue,
     capture_internal_exceptions,
+    ensure_integration_enabled,
     event_from_exception,
     logger,
     TimeoutThread,
+    reraise,
 )
-from sentry_sdk.integrations import Integration
-from sentry_sdk.integrations._wsgi_common import _filter_headers
 
 from sentry_sdk._types import TYPE_CHECKING
 
@@ -25,7 +27,6 @@ TIMEOUT_WARNING_BUFFER = 1.5  # Buffer time required to send timeout warning to
 MILLIS_TO_SECONDS = 1000.0
 
 if TYPE_CHECKING:
-    from datetime import datetime
     from typing import Any
     from typing import TypeVar
     from typing import Callable
@@ -38,16 +39,12 @@ if TYPE_CHECKING:
 
 def _wrap_func(func):
     # type: (F) -> F
+    @ensure_integration_enabled(GcpIntegration, func)
     def sentry_func(functionhandler, gcp_event, *args, **kwargs):
         # type: (Any, Any, *Any, **Any) -> Any
+        client = sentry_sdk.get_client()
 
-        hub = Hub.current
-        integration = hub.get_integration(GcpIntegration)
-        if integration is None:
-            return func(functionhandler, gcp_event, *args, **kwargs)
-
-        # If an integration is there, a client has to be there.
-        client = hub.client  # type: Any
+        integration = client.get_integration(GcpIntegration)
 
         configured_time = environ.get("FUNCTION_TIMEOUT_SEC")
         if not configured_time:
@@ -58,9 +55,9 @@ def _wrap_func(func):
 
         configured_time = int(configured_time)
 
-        initial_time = datetime_utcnow()
+        initial_time = datetime.now(timezone.utc)
 
-        with hub.push_scope() as scope:
+        with sentry_sdk.isolation_scope() as scope:
             with capture_internal_exceptions():
                 scope.clear_breadcrumbs()
                 scope.add_event_processor(
@@ -101,7 +98,7 @@ def _wrap_func(func):
                 },
                 "gcp_event": gcp_event,
             }
-            with hub.start_transaction(
+            with sentry_sdk.start_transaction(
                 transaction, custom_sampling_context=sampling_context
             ):
                 try:
@@ -113,13 +110,13 @@ def _wrap_func(func):
                         client_options=client.options,
                         mechanism={"type": "gcp", "handled": False},
                     )
-                    hub.capture_event(sentry_event, hint=hint)
+                    sentry_sdk.capture_event(sentry_event, hint=hint)
                     reraise(*exc_info)
                 finally:
                     if timeout_thread:
                         timeout_thread.stop()
                     # Flush out the event queue
-                    hub.flush()
+                    client.flush()
 
     return sentry_func  # type: ignore
 
@@ -155,10 +152,10 @@ def _make_request_event_processor(gcp_event, configured_timeout, initial_time):
     def event_processor(event, hint):
         # type: (Event, Hint) -> Optional[Event]
 
-        final_time = datetime_utcnow()
+        final_time = datetime.now(timezone.utc)
         time_diff = final_time - initial_time
 
-        execution_duration_in_millis = duration_in_milliseconds(time_diff)
+        execution_duration_in_millis = time_diff / timedelta(milliseconds=1)
 
         extra = event.setdefault("extra", {})
         extra["google cloud functions"] = {
@@ -188,7 +185,7 @@ def _make_request_event_processor(gcp_event, configured_timeout, initial_time):
         if hasattr(gcp_event, "headers"):
             request["headers"] = _filter_headers(gcp_event.headers)
 
-        if _should_send_default_pii():
+        if should_send_default_pii():
             if hasattr(gcp_event, "data"):
                 request["data"] = gcp_event.data
         else:
