@@ -1,23 +1,23 @@
-from __future__ import print_function
-
+from abc import ABC, abstractmethod
 import io
 import gzip
 import socket
 import time
-from datetime import timedelta
+import warnings
+from datetime import datetime, timedelta, timezone
 from collections import defaultdict
+from urllib.request import getproxies
 
 import urllib3
 import certifi
 
-from sentry_sdk.utils import Dsn, logger, capture_internal_exceptions, json_dumps
+from sentry_sdk.consts import EndpointType
+from sentry_sdk.utils import Dsn, logger, capture_internal_exceptions
 from sentry_sdk.worker import BackgroundWorker
 from sentry_sdk.envelope import Envelope, Item, PayloadRef
-from sentry_sdk._compat import datetime_utcnow
 from sentry_sdk._types import TYPE_CHECKING
 
 if TYPE_CHECKING:
-    from datetime import datetime
     from typing import Any
     from typing import Callable
     from typing import Dict
@@ -32,15 +32,10 @@ if TYPE_CHECKING:
     from urllib3.poolmanager import PoolManager
     from urllib3.poolmanager import ProxyManager
 
-    from sentry_sdk._types import Event, EndpointType
+    from sentry_sdk._types import Event
 
     DataCategory = Optional[str]
 
-try:
-    from urllib.request import getproxies
-except ImportError:
-    from urllib import getproxies  # type: ignore
-
 
 KEEP_ALIVE_SOCKET_OPTIONS = []
 for option in [
@@ -57,7 +52,7 @@ for option in [
         pass
 
 
-class Transport(object):
+class Transport(ABC):
     """Baseclass for all transports.
 
     A transport is used to send an event to sentry.
@@ -80,11 +75,23 @@ class Transport(object):
     ):
         # type: (...) -> None
         """
+        DEPRECATED: Please use capture_envelope instead.
+
         This gets invoked with the event dictionary when an event should
         be sent to sentry.
         """
-        raise NotImplementedError()
 
+        warnings.warn(
+            "capture_event is deprecated, please use capture_envelope instead!",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+
+        envelope = Envelope()
+        envelope.add_event(event)
+        self.capture_envelope(envelope)
+
+    @abstractmethod
     def capture_envelope(
         self, envelope  # type: Envelope
     ):
@@ -93,11 +100,10 @@ class Transport(object):
         Send an envelope to Sentry.
 
         Envelopes are a data container format that can hold any type of data
-        submitted to Sentry. We use it for transactions and sessions, but
-        regular "error" events should go through `capture_event` for backwards
-        compat.
+        submitted to Sentry. We use it to send all event data (including errors,
+        transactions, crons checkins, etc.) to Sentry.
         """
-        raise NotImplementedError()
+        pass
 
     def flush(
         self,
@@ -105,13 +111,23 @@ class Transport(object):
         callback=None,  # type: Optional[Any]
     ):
         # type: (...) -> None
-        """Wait `timeout` seconds for the current events to be sent out."""
-        pass
+        """
+        Wait `timeout` seconds for the current events to be sent out.
+
+        The default implementation is a no-op, since this method may only be relevant to some transports.
+        Subclasses should override this method if necessary.
+        """
+        return None
 
     def kill(self):
         # type: () -> None
-        """Forcefully kills the transport."""
-        pass
+        """
+        Forcefully kills the transport.
+
+        The default implementation is a no-op, since this method may only be relevant to some transports.
+        Subclasses should override this method if necessary.
+        """
+        return None
 
     def record_lost_event(
         self,
@@ -140,14 +156,26 @@ class Transport(object):
 def _parse_rate_limits(header, now=None):
     # type: (Any, Optional[datetime]) -> Iterable[Tuple[DataCategory, datetime]]
     if now is None:
-        now = datetime_utcnow()
+        now = datetime.now(timezone.utc)
 
     for limit in header.split(","):
         try:
-            retry_after, categories, _ = limit.strip().split(":", 2)
+            parameters = limit.strip().split(":")
+            retry_after, categories = parameters[:2]
+
             retry_after = now + timedelta(seconds=int(retry_after))
             for category in categories and categories.split(";") or (None,):
-                yield category, retry_after
+                if category == "metric_bucket":
+                    try:
+                        namespaces = parameters[4].split(";")
+                    except IndexError:
+                        namespaces = []
+
+                    if not namespaces or "custom" in namespaces:
+                        yield category, retry_after
+
+                else:
+                    yield category, retry_after
         except (LookupError, ValueError):
             continue
 
@@ -210,6 +238,7 @@ class HttpTransport(Transport):
                 # quantity of 0 is actually 1 as we do not want to count
                 # empty attachments as actually empty.
                 quantity = len(item.get_bytes()) or 1
+
         elif data_category is None:
             raise TypeError("data category not provided")
 
@@ -230,7 +259,7 @@ class HttpTransport(Transport):
         # sentries if a proxy in front wants to globally slow things down.
         elif response.status == 429:
             logger.warning("Rate-limited via 429")
-            self._disabled_until[None] = datetime_utcnow() + timedelta(
+            self._disabled_until[None] = datetime.now(timezone.utc) + timedelta(
                 seconds=self._retry.get_retry_after(response) or 60
             )
 
@@ -238,7 +267,7 @@ class HttpTransport(Transport):
         self,
         body,  # type: bytes
         headers,  # type: Dict[str, str]
-        endpoint_type="store",  # type: EndpointType
+        endpoint_type=EndpointType.ENVELOPE,  # type: EndpointType
         envelope=None,  # type: Optional[Envelope]
     ):
         # type: (...) -> None
@@ -336,14 +365,22 @@ class HttpTransport(Transport):
         # type: (str) -> bool
         def _disabled(bucket):
             # type: (Any) -> bool
+
+            # The envelope item type used for metrics is statsd
+            # whereas the rate limit category is metric_bucket
+            if bucket == "statsd":
+                bucket = "metric_bucket"
+
             ts = self._disabled_until.get(bucket)
-            return ts is not None and ts > datetime_utcnow()
+            return ts is not None and ts > datetime.now(timezone.utc)
 
         return _disabled(category) or _disabled(None)
 
     def _is_rate_limited(self):
         # type: () -> bool
-        return any(ts > datetime_utcnow() for ts in self._disabled_until.values())
+        return any(
+            ts > datetime.now(timezone.utc) for ts in self._disabled_until.values()
+        )
 
     def _is_worker_full(self):
         # type: () -> bool
@@ -353,46 +390,6 @@ class HttpTransport(Transport):
         # type: () -> bool
         return not (self._is_worker_full() or self._is_rate_limited())
 
-    def _send_event(
-        self, event  # type: Event
-    ):
-        # type: (...) -> None
-
-        if self._check_disabled("error"):
-            self.on_dropped_event("self_rate_limits")
-            self.record_lost_event("ratelimit_backoff", data_category="error")
-            return None
-
-        body = io.BytesIO()
-        if self._compresslevel == 0:
-            body.write(json_dumps(event))
-        else:
-            with gzip.GzipFile(
-                fileobj=body, mode="w", compresslevel=self._compresslevel
-            ) as f:
-                f.write(json_dumps(event))
-
-        assert self.parsed_dsn is not None
-        logger.debug(
-            "Sending event, type:%s level:%s event_id:%s project:%s host:%s"
-            % (
-                event.get("type") or "null",
-                event.get("level") or "null",
-                event.get("event_id") or "null",
-                self.parsed_dsn.project_id,
-                self.parsed_dsn.host,
-            )
-        )
-
-        headers = {
-            "Content-Type": "application/json",
-        }
-        if self._compresslevel > 0:
-            headers["Content-Encoding"] = "gzip"
-
-        self._send_request(body.getvalue(), headers=headers)
-        return None
-
     def _send_envelope(
         self, envelope  # type: Envelope
     ):
@@ -402,7 +399,7 @@ class HttpTransport(Transport):
         new_items = []
         for item in envelope.items:
             if self._check_disabled(item.data_category):
-                if item.data_category in ("transaction", "error", "default"):
+                if item.data_category in ("transaction", "error", "default", "statsd"):
                     self.on_dropped_event("self_rate_limits")
                 self.record_lost_event("ratelimit_backoff", item=item)
             else:
@@ -450,7 +447,7 @@ class HttpTransport(Transport):
         self._send_request(
             body.getvalue(),
             headers=headers,
-            endpoint_type="envelope",
+            endpoint_type=EndpointType.ENVELOPE,
             envelope=envelope,
         )
         return None
@@ -540,23 +537,6 @@ class HttpTransport(Transport):
         else:
             return urllib3.PoolManager(**opts)
 
-    def capture_event(
-        self, event  # type: Event
-    ):
-        # type: (...) -> None
-        hub = self.hub_cls.current
-
-        def send_event_wrapper():
-            # type: () -> None
-            with hub:
-                with capture_internal_exceptions():
-                    self._send_event(event)
-                    self._flush_client_reports()
-
-        if not self._worker.submit(send_event_wrapper):
-            self.on_dropped_event("full_queue")
-            self.record_lost_event("queue_overflow", data_category="error")
-
     def capture_envelope(
         self, envelope  # type: Envelope
     ):
@@ -594,6 +574,11 @@ class HttpTransport(Transport):
 
 
 class _FunctionTransport(Transport):
+    """
+    DEPRECATED: Users wishing to provide a custom transport should subclass
+    the Transport class, rather than providing a function.
+    """
+
     def __init__(
         self, func  # type: Callable[[Event], None]
     ):
@@ -608,19 +593,33 @@ class _FunctionTransport(Transport):
         self._func(event)
         return None
 
+    def capture_envelope(self, envelope: Envelope) -> None:
+        # Since function transports expect to be called with an event, we need
+        # to iterate over the envelope and call the function for each event, via
+        # the deprecated capture_event method.
+        event = envelope.get_event()
+        if event is not None:
+            self.capture_event(event)
+
 
 def make_transport(options):
     # type: (Dict[str, Any]) -> Optional[Transport]
     ref_transport = options["transport"]
 
-    # If no transport is given, we use the http transport class
-    if ref_transport is None:
-        transport_cls = HttpTransport  # type: Type[Transport]
-    elif isinstance(ref_transport, Transport):
+    # By default, we use the http transport class
+    transport_cls = HttpTransport  # type: Type[Transport]
+
+    if isinstance(ref_transport, Transport):
         return ref_transport
     elif isinstance(ref_transport, type) and issubclass(ref_transport, Transport):
         transport_cls = ref_transport
     elif callable(ref_transport):
+        warnings.warn(
+            "Function transports are deprecated and will be removed in a future release."
+            "Please provide a Transport instance or subclass, instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         return _FunctionTransport(ref_transport)
 
     # if a transport class is given only instantiate it if the dsn is not
